@@ -5,7 +5,7 @@
 ;; Author: Feng Shu <tumashu@163.com>
 ;; Maintainer: Feng Shu <tumashu@163.com>
 ;; URL: https://github.com/tumashu/vertico-posframe
-;; Version: 0.2.0
+;; Version: 0.4.1
 ;; Keywords: abbrev, convenience, matching, vertico
 ;; Package-Requires: ((emacs "26.0") (posframe "1.0.0") (vertico "0.13.0"))
 
@@ -56,12 +56,11 @@
 
 ;;; Code:
 ;; * vertico-posframe's code
-(require 'cl-lib)
 (require 'posframe)
 (require 'vertico)
 
 (defgroup vertico-posframe nil
-  "Using posframe to show vertico"
+  "Using posframe to show vertico."
   :group 'vertico-posframe)
 
 (defcustom vertico-posframe-font nil
@@ -108,6 +107,19 @@ When 0, no border is showed."
   "The frame parameters used by vertico-posframe."
   :type 'string)
 
+(defcustom vertico-posframe-show-minibuffer-rules
+  (list "^eval-*")
+  "A list of rule showed minibuffer.
+
+a rule can be a regexp or a function.
+
+1. when rule is a regexp and it match `this-command'.
+2. when rule is a function and it return t.
+3. when rule is a symbol, its value is t.
+
+minibuffer will not be hided by minibuffer-cover."
+  :type '(repeat (choice string function)))
+
 (defface vertico-posframe
   '((t (:inherit default)))
   "Face used by the vertico-posframe."
@@ -124,17 +136,17 @@ When 0, no border is showed."
   :group 'vertico-posframe)
 
 (defvar vertico-posframe--buffer " *vertico-posframe--buffer*")
+(defvar vertico-posframe--minibuffer-cover " *vertico-posframe--minibuffer-cover*")
 (defvar vertico-posframe--last-window nil)
 
-(defvar vertico-posframe--overlay)
-
 ;; Fix warn
 (defvar exwm--connection)
 (defvar exwm-workspace--workareas)
 (defvar exwm-workspace-current-index)
 
 (defun vertico-posframe-refposhandler-default (&optional frame)
-  "The default posframe refposhandler used by vertico-posframe."
+  "The default posframe refposhandler used by vertico-posframe.
+Optional argument FRAME ."
   (cond
    ;; EXWM environment
    ((bound-and-true-p exwm--connection)
@@ -169,33 +181,92 @@ When 0, no border is showed."
 
 (defun vertico-posframe--display (lines)
   "Display LINES in posframe."
-  (let ((count (vertico--format-count))
-        (prompt (minibuffer-prompt))
-        (content (minibuffer-contents)))
+  (let ((count (vertico-posframe--format-count))
+        (prompt (propertize (minibuffer-prompt) 'face 'minibuffer-prompt))
+        (content (minibuffer-contents))
+        (show-minibuffer (vertico-posframe--show-minibuffer-p)))
     (with-current-buffer (get-buffer-create vertico-posframe--buffer)
-      (add-hook 'window-selection-change-functions 'vertico-posframe--select nil 'local)
       (setq-local inhibit-modification-hooks t
                   cursor-in-non-selected-windows 'box)
       (erase-buffer)
-      (insert (propertize (concat count prompt) 'face 'minibuffer-prompt)
-              content
-              (propertize " " 'face 'vertico-posframe-cursor)
-              "\n" (string-join lines)))
+      (insert count prompt content " \n" (string-join lines)))
     (with-selected-window (vertico-posframe-last-window)
-      (apply #'posframe-show
-             vertico-posframe--buffer
-             :font vertico-posframe-font
-             :poshandler vertico-posframe-poshandler
-             :background-color (face-attribute 'vertico-posframe :background nil t)
-             :foreground-color (face-attribute 'vertico-posframe :foreground nil t)
-             :border-width vertico-posframe-border-width
-             :border-color (face-attribute 'vertico-posframe-border :background nil t)
-             :override-parameters vertico-posframe-parameters
-             :refposhandler vertico-posframe-refposhandler
-             :hidehandler #'vertico-posframe-hidehandler
-             :lines-truncate t
-             (funcall vertico-posframe-size-function))
-      (setq vertico-posframe--last-window nil))))
+      ;; Create a posframe to cover minibuffer.
+      (if show-minibuffer
+          (vertico-posframe--hide-minibuffer-cover)
+        (vertico-posframe--create-minibuffer-cover))
+      (vertico-posframe--show))))
+
+(defun vertico-posframe--format-count ()
+  "Format vertico count."
+  (propertize (or (vertico--format-count) "") 'face 'minibuffer-prompt))
+
+(defun vertico-posframe--show (&optional string)
+  "`posframe-show' of vertico-posframe.
+Show STRING when it is a string."
+  (apply #'posframe-show
+         vertico-posframe--buffer
+         :string string
+         :font vertico-posframe-font
+         :poshandler vertico-posframe-poshandler
+         :background-color (face-attribute 'vertico-posframe :background nil t)
+         :foreground-color (face-attribute 'vertico-posframe :foreground nil t)
+         :border-width vertico-posframe-border-width
+         :border-color (face-attribute 'vertico-posframe-border :background nil t)
+         :override-parameters vertico-posframe-parameters
+         :refposhandler vertico-posframe-refposhandler
+         :hidehandler #'vertico-posframe-hidehandler
+         :lines-truncate t
+         (funcall vertico-posframe-size-function)))
+
+(defun vertico-posframe--show-init ()
+  "Create posframe in advance to limit flicker for `vertico-posframe--show'."
+  (posframe-show vertico-posframe--buffer
+                 :string ""
+                 :font vertico-posframe-font
+                 :position (cons 0 0)
+                 :background-color (face-attribute 'vertico-posframe :background nil t)
+                 :foreground-color (face-attribute 'vertico-posframe :foreground nil t)
+                 :border-width vertico-posframe-border-width
+                 :border-color (face-attribute 'vertico-posframe-border :background nil t)
+                 :override-parameters vertico-posframe-parameters
+                 :timeout 0.1))
+
+(defun vertico-posframe--create-minibuffer-cover (&optional string)
+  "Create minibuffer cover."
+  (let ((color (face-background 'default nil))
+        (win (active-minibuffer-window)))
+    (posframe-show vertico-posframe--minibuffer-cover
+                   :string (or string (make-string (frame-width) ?\ ))
+                   :position (cons 0 (- (frame-pixel-height) (window-pixel-height win)))
+                   :height (+ (window-height win) 1)
+                   :background-color color
+                   :foreground-color color
+                   :lines-truncate t
+                   :timeout 3)))
+
+(defun vertico-posframe--hide-minibuffer-cover ()
+  "Hide minibuffer cover."
+  ;; FIXME: delay 0.1 second to remove minibuffer cover, which can
+  ;; limit minibuffer flicker.
+  (run-with-timer
+   0.1 nil
+   (lambda ()
+     (posframe-hide vertico-posframe--minibuffer-cover))))
+
+(defun vertico-posframe--show-minibuffer-p ()
+  "Test show minibuffer or not."
+  (or current-input-method
+      (cl-some
+       (lambda (rule)
+         (cond ((functionp rule)
+                (funcall rule))
+               ((and rule (stringp rule))
+                (string-match-p rule (symbol-name this-command)))
+               ((symbolp rule)
+                (symbol-value rule))
+               (t nil)))
+       vertico-posframe-show-minibuffer-rules)))
 
 (defun vertico-posframe-last-window ()
   "Get the last actived window before active minibuffer."
@@ -205,33 +276,59 @@ When 0, no border is showed."
           (next-window))
         (selected-window))))
 
-(defun vertico-posframe--select (_)
-  "Ensure that cursor is only shown if minibuffer is selected."
-  (with-current-buffer (buffer-local-value 'vertico-posframe--buffer
-                                           (window-buffer (active-minibuffer-window)))
-    (if (eq (selected-window) (active-minibuffer-window))
-        (setq-local cursor-in-non-selected-windows 'box)
-      (setq-local cursor-in-non-selected-windows nil)
-      (goto-char (point-min)))))
-
 (defun vertico-posframe--hide ()
   "Hide vertico buffer."
   (when (posframe-workable-p)
-    (posframe-hide vertico-posframe--buffer)))
+    (posframe-hide vertico-posframe--buffer)
+    (vertico-posframe--hide-minibuffer-cover)))
+
+(defun vertico-posframe--post-command-function ()
+  "`post-command-hook' function used by vertico-posframe."
+  (while-no-input
+    (redisplay)
+    (when (and vertico-posframe-mode
+               (not (minibufferp)))
+      (vertico-posframe--hide-minibuffer-cover))
+    (when (and vertico-posframe-mode
+               (minibufferp)
+               (posframe-workable-p))
+      (with-current-buffer (window-buffer (active-minibuffer-window))
+        (let* ((point (point))
+               (count (vertico-posframe--format-count))
+               ;; NOTE: Vertico count in minibuffer is before-string
+               ;; of an overlay, so the result of `buffer-string' will
+               ;; not include it.
+               (contents (buffer-string))
+               (n (+ point (length count)))
+               (cursor-face
+                ;; FIXME: make sure background and foreground do
+                ;; not have similar color. ivy-posframe have not
+                ;; this problem, I can not find the reason.
+                (list :foreground (face-attribute 'default :background)
+                      :inherit 'vertico-posframe-cursor)))
+          (remove-text-properties 0 (length contents) '(read-only nil) contents)
+          (with-current-buffer (get-buffer-create vertico-posframe--buffer)
+            (goto-char (point-min))
+            (delete-region (point) (line-beginning-position 2))
+            (insert count contents "  \n")
+            (add-text-properties n (+ n 1) `(face ,cursor-face))))))))
 
 (defun vertico-posframe--setup ()
   "Setup minibuffer overlay, which pushes the minibuffer content down."
-  (add-hook 'window-selection-change-functions 'vertico-posframe--select nil 'local)
   (add-hook 'minibuffer-exit-hook 'vertico-posframe--hide nil 'local)
-  (setq-local cursor-type '(bar . 0))
-  (setq vertico-posframe--overlay (make-overlay (point-max) (point-max) nil t t))
-  (overlay-put vertico-posframe--overlay 'window (selected-window))
-  (overlay-put vertico-posframe--overlay 'priority 1000)
-  (overlay-put vertico-posframe--overlay 'before-string "\n"))
+  (setq-local cursor-type '(bar . 0)))
 
 (defun vertico-posframe--advice (&rest _args)
   "Advice for ORIG completion function, receiving ARGS."
-  (setq vertico-posframe--last-window (selected-window)))
+  (unless (minibufferp) ; minibuffer recursive
+    (setq vertico-posframe--last-window (selected-window))))
+
+(defun vertico-posframe--minibuffer-message (message &rest _args)
+  "Advice function of `minibuffer-message'.
+Argument MESSAGE ."
+  (let* ((count (vertico-posframe--format-count))
+         (contents (buffer-string)))
+    (vertico-posframe--show (concat count contents message))))
 
 ;;;###autoload
 (define-minor-mode vertico-posframe-mode
@@ -239,16 +336,24 @@ When 0, no border is showed."
   :global t
   (cond
    (vertico-posframe-mode
+    (advice-add #'minibuffer-message :before #'vertico-posframe--minibuffer-message)
     (advice-add #'vertico--display-candidates :override #'vertico-posframe--display)
     (advice-add #'vertico--setup :after #'vertico-posframe--setup)
     (advice-add #'completing-read-default :before #'vertico-posframe--advice)
-    (advice-add #'completing-read-multiple :before #'vertico-posframe--advice))
+    (advice-add #'completing-read-multiple :before #'vertico-posframe--advice)
+    (add-hook 'post-command-hook #'vertico-posframe--post-command-function)
+    ;; Create posframe in advance to limit flicker.
+    (vertico-posframe--show-init)
+    (vertico-posframe--create-minibuffer-cover ""))
    (t
+    (advice-remove #'minibuffer-message #'vertico-posframe--minibuffer-message)
     (advice-remove #'vertico--display-candidates #'vertico-posframe--display)
     (advice-remove #'vertico--setup #'vertico-posframe--setup)
     (advice-remove #'completing-read-default #'vertico-posframe--advice)
     (advice-remove #'completing-read-multiple #'vertico-posframe--advice)
-    (posframe-delete vertico-posframe--buffer))))
+    (remove-hook 'post-command-hook #'vertico-posframe--post-command-function)
+    (posframe-delete vertico-posframe--buffer)
+    (posframe-delete vertico-posframe--minibuffer-cover))))
 
 (provide 'vertico-posframe)
 ;;; vertico-posframe.el ends here
